# FileMorph: Rename files, edit attributes

#Copyright (C) 2016 WordTech Communications LLC

#MIT license

#let's get started

use FindBin qw($Bin);
use Config;
use File::Spec::Functions;
use File::Copy::Recursive qw(dircopy );
use Env qw(HOME);
use File::Path qw(remove_tree);
use File::Copy::Recursive qw (rmove);

BEGIN {

    if ( $Config{osname} eq "darwin" ) {
        my $frameworkpath = "$Bin/Contents/Frameworks/Tcl.framework/Tcl";
        $ENV{'PERL_TCL_DL_PATH'} = $frameworkpath;
    }
    if ( $Config{osname} eq "MSWin32" ) {
        my $frameworkpath = "$Bin/Tcl/bin/tcl86.dll";
        $ENV{'PERL_TCL_DL_PATH'} = $frameworkpath;
    }

}

#use strict;
use warnings;

use Tkx;

our $IS_AQUA    = &Tkx::tk_windowingsystem eq "aqua";
our $IS_WINDOWS = &Tkx::tk_windowingsystem eq "win32";

use File::Copy qw(move copy);
use Env qw (HOME TMP TEMP TMPDIR);
use Time::Local;
use POSIX qw( strftime );
use File::Basename qw(fileparse basename dirname);
use LWP::Simple;
use Config::IniFiles;
use File::Temp qw(tempfile);

Tkx::set( '::argv0', $ARGV[0] || "" );
Tkx::catch('console hide');

Tkx::package_require('tkdnd');
Tkx::package_require('Img');
Tkx::package_require('regproc');
Tkx::package_require('machelp');
Tkx::package_require('softwareupdate');
Tkx::package_require('aquahig');
Tkx::package_require('tablelist_tile');
Tkx::package_require('BWidget');
Tkx::package_require('widget::dateentry');
Tkx::package_require('xplat');

if ($IS_AQUA) {
    Tkx::package_require('fullscreen');
    Tkx::package_require('aem');
    Tkx::package_require('cocoaprint');
    Tkx::package_require('windowlist');
}

#initialize variables
our $folder;
our $sourceext;
our $targetext;
our $targetstring;
our $srcstring;
our $filename;
our $prependstring;
our $prepend;
our @filelist;
our $appname;
our $appversion;
our $datadir;
our $prefdir;
our $appicon;
our $datetime;
our $clocktime;
our $modtime;
our $timeofday;
our $icon;
our $fileimage;
our $folderimage;

$datetime = strftime( "% m/%d/ % Y", localtime );

#initialize the app
&startUp;
&makeImage;

@filelist = ();

if ($IS_AQUA) {
    Tkx::interp_alias( "", "::tk::mac::OpenDocument", "", [ \&openDoc ] );
    Tkx::interp_alias( "", "::tk::mac::ShowHelp",     "", [ \&showHelp ] );
    Tkx::interp_alias( "", "bgerror",                 "", [ \&errlog ] );
}

if ($IS_WINDOWS) {
    Tkx::package_require("twapi");
}

#draw the GUI
our $mw = Tkx::widget->new(".");
Tkx::wm_title( $mw, "FileMorph" );
$mw->g_wm_withdraw;
if ($IS_WINDOWS) {
    $mw->g_wm_protocol( WM_DELETE_WINDOW => \&shutdown );
    $mw->g_wm_iconphoto( -default => $icon );
}

#menubar code
my $menubar = $mw->new_menu();
Tkx::option_add( "*tearOff", 0 );
our $applemenu;
if ($IS_AQUA) {
    $applemenu = $menubar->new_menu( -name => "apple" );
}
my $searchmenu = $menubar->new_menu();
my $helpmenu = $menubar->new_menu( -name => "help" );

if ($IS_AQUA) {
    $menubar->add_cascade(
        -label => "FileMorph",
        -menu  => $applemenu,
    );

    $applemenu->add_command(
        -label   => "About FileMorph",
        -command => \&Tkx::tkAboutDialog,
    );

    $applemenu->add_command(
        -label   => "License",
        -command => \&Tkx::regproc__getReg
    );
    $applemenu->add_command(
        -label   => "Check for Updates",
        -command => \&checkUpdate,
    );

    $applemenu->add_separator();
}

$menubar->add_cascade(
    -label => "Directory",
    -menu  => $searchmenu,
);

$searchmenu->add_command(
    -label   => "Select Directory",
    -command => \&setDir,
);
$searchmenu->add_command(
    -label   => "Select File",
    -command => \&setFile,
);

if ($IS_WINDOWS) {
    $searchmenu->add_separator;
    $searchmenu->add_command(
        -label   => "Exit",
        -command => \&shutdown,
    );
}

if ($IS_AQUA) {
    Tkx::windowlist__windowMenu($menubar);
}

$menubar->add_cascade(
    -label => "Help",
    -menu  => $helpmenu,
);

if ($IS_WINDOWS) {

    $helpmenu->add_command(
        -label   => "FileMorph Help",
        -command => \&Tkx::machelp__userhelp,
    );
}
$helpmenu->add_command(
    -label   => "Contact Code by Kevin",
    -command => sub {
        Tkx::xplat__launch('mailto:kw@codebykevin.com?subject=FileMorph');
    }
);

$helpmenu->add_command(
    -label   => "Web Site",
    -command => sub {
        Tkx::xplat__launch('http://www.codebykevin.com/filemorph.html');
    }
);

if ($IS_WINDOWS) {
    $helpmenu->add_command(
        -label   => "About FileMorph",
        -command => \&Tkx::tkAboutDialog,
    );

    $helpmenu->add_command(
        -label   => "License",
        -command => \&Tkx::regproc__getReg
    );
    $helpmenu->add_command(
        -label   => "Check for Updates",
        -command => \&checkUpdate,
    );
}

$mw->configure( -menu => $menubar );

#bindings
if ($IS_AQUA) {
    $mw->g_bind( "<Command-W>", sub { $mw->g_wm_state("withdrawn"); } );
    $mw->g_bind( "<Command-w>", sub { $mw->g_wm_state("withdrawn"); } );
}

#main window frame, listbox and notebook

my $toolbar = $mw->new_ttk__frame( -padding => 5 );
$toolbar->g_pack(
    -fill   => "both",
    -expand => "no",
    -side   => "top",
);

my $folderbutton = $toolbar->new_ttk__button(
    -image     => $folderimage,
    -style     => "Toolbutton",
    -takefocus => 0,
    -command   => \&setDir,
);
$folderbutton->g_pack(
    -expand => "no",
    -fill   => "both",
    -side   => "left"
);

Tkx::DynamicHelp__register( $folderbutton, "balloon", "Select the directory" );

my $filebutton = $toolbar->new_ttk__button(
    -image     => $fileimage,
    -style     => "Toolbutton",
    -takefocus => 0,
    -command   => \&setFile,
);

$filebutton->g_pack(
    -expand => "no",
    -fill   => "both",
    -side   => "left"
);

Tkx::DynamicHelp__register( $filebutton, "balloon", "Select the file" );

my $sep = $toolbar->new_ttk__separator( -orient => "vertical" );
$sep->g_pack(
    -fill   => "both",
    -expand => "no",
    -side   => "top",
);

my $frame = $mw->new_ttk__frame;

$frame->g_pack(
    -fill   => "both",
    -expand => "yes",
    -side   => "bottom",
);

my $topsep = $frame->new_ttk__separator( -orient => "vertical" );
$topsep->g_pack(
    -fill   => "both",
    -expand => "no",
    -side   => "top",
);

my $textframe = $frame->new_ttk__panedwindow( -orient => "horizontal", );

$textframe->g_pack(
    -fill   => "both",
    -expand => "yes",
    -side   => "top",
);

my $listframe = $textframe->new_ttk__frame;
$listframe->g_pack(
    -fill   => "both",
    -expand => "yes",
);

my $header = $listframe->new_ttk__label(
    -text  => "File",
    -style => "TablelistHeader.TLabel",
);
$header->g_pack(
    -fill   => "both",
    -expand => "yes",
    -side   => "top",
);

my $boxframe = $listframe->new_ttk__frame;
$boxframe->g_pack(
    -fill   => "both",
    -expand => "yes",
    -side   => "top",
);

my $listbox = $boxframe->new_listbox(
    -relief           => "flat",
    -bd               => 0,
    -selectbackground => "RoyalBlue1",
    -selectforeground => "white",
    -activestyle      => "none",
    -height           => 100,
);
$listbox->g_pack(
    -fill   => "both",
    -expand => "yes",
    -side   => "left",
);

Tkx::tkdnd__drop___target_register( $listbox, "*" );
Tkx::bind( $listbox, "<<Drop:DND_Files>>",
    [ sub { &selectFileSystem; }, Tkx::Ev("%D") ] );

my $noteframe = $textframe->new_ttk__frame;
$noteframe->g_pack(
    -fill   => "both",
    -expand => "yes",
    -side   => "right",
);

my $notebook = $noteframe->new_ttk__notebook;
$notebook->g_pack(
    -fill   => "both",
    -expand => "yes",
    -side   => "right",
);

my $scrollhouse = $boxframe->new_ttk__frame;
$scrollhouse->g_pack(
    -fill   => "y",
    -expand => "no",
    -side   => "right",
);

my $scrollbar = $scrollhouse->new_ttk__scrollbar( -orient => 'vertical' );
$scrollbar->configure( -command => [ $listbox, "yview" ] );
$scrollbar->g_pack(
    -fill   => "y",
    -expand => "no",
    -side   => "right",
);

$listbox->configure( -yscrollcommand => [ $scrollbar, "set" ] );

#notebook components
my $extension   = $notebook->new_ttk__frame;
my $replacement = $notebook->new_ttk__frame;
my $text        = $notebook->new_ttk__frame;
my $date        = $notebook->new_ttk__frame;
my $case        = $notebook->new_ttk__frame;
my $attributes  = $notebook->new_ttk__frame;

#UI to modify file extensions
$notebook->add( $extension, -text => "Extension" );
my $exsrclabel = $extension->new_ttk__label( -text => "Extension:" );
my $exsrcentry = $extension->new_ttk__entry( -textvar => \$sourceext );
my $extargetlabel =
  $extension->new_ttk__label( -text => "New Extension Name:" );
my $extargetentry = $extension->new_ttk__entry( -textvar => \$targetext );
my $extchange = $extension->new_ttk__button(
    -text    => "Change",
    -command => \&changeExtension
);
my $extundo = $extension->new_ttk__button(
    -text    => "Undo",
    -command => \&undoExtension
);
my $extclear = $extension->new_ttk__button(
    -text    => "Clear",
    -command => sub { $sourceext = ""; $targetext = ""; }
);

our @extarray = (
    $exsrclabel, $exsrcentry, $extargetlabel, $extargetentry,
    $extchange, $extundo, $extclear
);

foreach (@extarray) {
    $_->g_pack(
        -fill   => "both",
        -expand => "no",
        -side   => "top",
    );
}

#UI to replace text in file name
$notebook->add( $replacement, -text => "Replace Text" );
my $repsrclabel = $replacement->new_ttk__label( -text => "Text:" );
my $repsrcentry = $replacement->new_ttk__entry( -textvar => \$srcstring );
my $reptargetlabel =
  $replacement->new_ttk__label( -text => "Replacement Text:" );
my $reptargetentry = $replacement->new_ttk__entry( -textvar => \$targetstring );
my $repbutton = $replacement->new_ttk__button(
    -text    => "Change",
    -command => \&replaceString
);
my $repundo = $replacement->new_ttk__button(
    -text    => "Undo",
    -command => \&undoReplaceString
);
my $repclear = $replacement->new_ttk__button(
    -text    => "Clear",
    -command => sub { $srcstring = ""; $targetstring = ""; },
);

our @reparray = (
    $repsrclabel, $repsrcentry, $reptargetlabel, $reptargetentry,
    $repbutton, $repundo, $repclear
);

foreach (@reparray) {
    $_->g_pack(
        -fill   => "both",
        -expand => "no",
        -side   => "top",
    );
}

#UI to add text to file name
$notebook->add( $text, -text => "Add Text" );
my $txtsrclabel = $text->new_ttk__label( -text => "Text:" );
my $txtsrcentry = $text->new_ttk__entry( -textvar => \$prependstring );
my $txtbefore = $text->new_ttk__radiobutton(
    -text     => "Before",
    -variable => \$prepend,
    -value    => 0
);
my $txtafter = $text->new_ttk__radiobutton(
    -text     => "After",
    -variable => \$prepend,
    -value    => 1
);
my $txtbutton =
  $text->new_ttk__button( -text => "Change", -command => \&addString );
my $txtundo = $text->new_ttk__button(
    -text    => "Undo",
    -command => \&undoAddString
);
my $txtclear = $text->new_ttk__button(
    -text    => "Clear",
    -command => sub { $prependstring = ""; }
);

$prepend = 1;

our @txtarray = (
    $txtsrclabel, $txtsrcentry, $txtbefore, $txtafter,
    $txtbutton,   $txtundo,     $txtclear
);

foreach (@txtarray) {
    $_->g_pack(
        -fill   => "both",
        -expand => "no",
        -side   => "top",
    );
}

#UI to modify file attributes
$notebook->add( $attributes, -text => "Date" );

my $todaylabel = $attributes->new_ttk__label( -text => "Date:", );
my $today = $attributes->new_widget__dateentry(
    -textvariable => \$datetime,
    -font         => \&Tkx::TkDefaultFont
);
my $timelabel = $attributes->new_ttk__label(
    -text => "Time (enter HH:MM, military-style, i.e. 23:00 for 11:00 PM)", );

my $timeentry = $attributes->new_ttk__entry( -textvariable => \$clocktime, );
my $attbutton = $attributes->new_ttk__button(
    -text    => "Change",
    -command => \&changeTime
);

our @attarray = ( $todaylabel, $today, $timelabel, $timeentry, $attbutton );

foreach (@attarray) {
    $_->g_pack(
        -fill   => "both",
        -expand => "no",
        -side   => "top",
    );
}

$textframe->add($listframe);
$textframe->add($noteframe);

$mw->g_wm_geometry("900x800");
$mw->g_wm_deiconify;

if ($IS_AQUA) {
    Tkx::fullscreen__fullscreen($mw);
}

Tkx::after(50);

Tkx::regproc__readLic( $appname, $appversion );

#This is necessary to get the listbox to draw; sometimes it does not render until the mouse hovers over it
Tkx::focus( -force => $listbox );

if ($IS_AQUA) {
    &createAppleScriptCommands;
}

Tkx::MainLoop();
exit;

###----subroutines

sub shutdown {
    &Tkx::exit;
}

sub makeImage {

    my $data = "iVBORw0KGgoAAAANSUhEUgAAAEAAAABACAYAAACqaXHeAAAABGdBTUEAALGPC/xh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";
    $icon = Tkx::image_create_photo( -data => $data );

    my $folderdata = "
                R0lGODlhGAAYAPABAAAAAP///yH5BAEKAAEAIf4aQ3JlYXRlZCB3aXRoIEdJTVAg
b24gYSBNYWMALAAAAAAYABgAAAI1hI8Wy70JgZshJurOtFzmHm0ghJXVZVJkKo7u
C8cr28z0g944ojP2/aMFWcNU0SRLxk5KSQEAOw==
";
    $folderimage = Tkx::image_create_photo( -data => $folderdata );

    my $filedata = "
                R0lGODlhEgAYAPABAAAAAP///yH5BAEKAAEAIf4aQ3JlYXRlZCB3aXRoIEdJTVAg
b24gYSBNYWMALAAAAAASABgAAAI1hI8Qy5GvmoNJHjkv0+1tg0EL93nkSJ1ZioVV
270wisyl6tb2Cu6eL5v9bENhEJZKKpdMQwEAOw==
                ";
    $fileimage = Tkx::image_create_photo( -data => $filedata );

}

#check filelist for error handling
sub checkfilelist {

    if ( scalar(@filelist) == 0 ) {

        errlog("Please select a file or directory to run this operation.");

    }
}

#"about application"command
sub Tkx::tkAboutDialog {

    our $about = $mw->new_toplevel;
    Tkx::wm_title( $about, "About FileMorph" );

    my $aboutframe  = $about->new_frame;
    my $buttonframe = $about->new_frame;

    $aboutframe->g_pack(
        -fill   => "both",
        -expand => "yes",
        -side   => "top",
    );

    $buttonframe->g_pack(
        -fill   => "both",
        -expand => "yes",
        -side   => "bottom",
    );

    my $logolabel = $aboutframe->new_label(
        -image => $icon,
        -bg    => "gray95",
        -pady  => "2"
    );

    $logolabel->g_pack(
        -fill   => "both",
        -expand => "yes",
        -side   => "top",
    );

    my $label1 = $aboutframe->new_label(
        -text => "FileMorph $appversion",
        -font => "-weight bold",
        -pady => "2"
    );

    $label1->g_pack(
        -fill   => "both",
        -expand => "no",
        -side   => "top",
    );

    my $label2 = $aboutframe->new_label(
        -text => "Easy File Modification",
        -pady => "2"
    );

    $label2->g_pack(
        -fill   => "both",
        -expand => "no",
        -side   => "top",
    );

    my $label3 = $aboutframe->new_label(
        -text => "Copyright (c) 2016 WordTech Communications LLC",
        -pady => "5",
    );

    $label3->g_pack(
        -fill   => "both",
        -expand => "no",
        -side   => "top",
    );

    my $label4 = $aboutframe->new_label(
        -text => "http://www.codebykevin.com/FileMorph.html",
        -fg   => "blue",
        -pady => "2"
    );

    $label4->g_pack(
        -fill   => "both",
        -expand => "no",
        -side   => "top",
    );

    my $label5 = $aboutframe->new_label(
        -text =>
"See Acknowledgements in FileMorph Help\nfor additional credits\nand see History in FileMorph Help\nfor changes in this version",
        -pady => "2",
    );

    $label5->g_pack(
        -fill   => "both",
        -expand => "no",
        -side   => "top",
    );

    $label4->g_bind(
        "<Button-1>",
        sub {
            Tkx::xplat__launch("http://www.codebykevin.com/filemorph.html");
        }
    );

}

#handle errors in Tk
sub errlog {
    my $msg = shift;
    if ($IS_AQUA) {
        system("syslog -s -l Error FileMorph: An error occurred: $msg\n");
    }
    else {
        print "An error occurred: $msg\n";
    }

}

#error message for selected file
sub fileErrMsg {

    errlog(
"Cannot run operation on file. This operation can only be run on all files in a directory, not an individual file. Please choose a directory for batch operation."
    );
}

#error message for missing data
sub dataErrMsg {

    errlog(
"Cannot run operation without text.Please enter text into the appropriate entry field to run this operation."
    );
}

#update display of files
sub updateFileDisplay {

    $listbox->delete( 0, "end" );
    if ( scalar(@filelist) > 1 ) {
        @filelist = <"$folder/*">;
    }
    foreach (@filelist) {
        $listbox->insert( "end", "$_" );
    }
}

#set search directory
sub setDir {
    $listbox->delete( 0, "end" );
    $folder = Tkx::tk___chooseDirectory( -parent => $mw );
    return unless $folder;
    @filelist = <"$folder/*">;
    foreach (@filelist) {
        $listbox->insert( "end", "$_" );
    }
    return "$folder";
}

#set file to change
sub setFile {

    $listbox->delete( 0, "end" );
    my $filename = Tkx::tk___getOpenFile( -parent => $mw );
    return unless $filename;
    @filelist = ();
    push( @filelist, $filename );
    foreach (@filelist) {
        $listbox->insert( "end", "$_" );
    }
    return @filelist;
}

#change all file extensions in a directory
sub changeExtension {

    &checkfilelist;

    return if ( scalar(@filelist) == 0 );

    if ( scalar(@filelist) == 1 ) {

        &fileErrMsg;
        return;
    }

    if ( $exsrcentry->get() eq "" || $extargetentry->get() eq "" ) {
        &dataErrMsg;
        return;
    }

    $sourceext = $exsrcentry->get();
    $targetext = $extargetentry->get();

    foreach (@filelist) {
        my $filename = $_;

        my $newfilename = $filename;
        $newfilename =~ s/\.$sourceext$/\.$targetext/i;

        move $filename, $newfilename;
    }

    &updateFileDisplay;
}

#undo operation of change file extension
sub undoExtension {

    &checkfilelist;

    return if ( scalar(@filelist) == 0 );

    if ( scalar(@filelist) == 1 ) {

        &fileErrMsg;
        return;
    }

    if ( $exsrcentry->get() eq "" || $extargetentry->get() eq "" ) {
        &dataErrMsg;
        return;
    }

    $sourceext = $exsrcentry->get();
    $targetext = $extargetentry->get();

    foreach (@filelist) {
        next unless -f $_;
        my $filename    = $_;
        my $newfilename = $filename;
        $newfilename =~ s/\.$targetext$/.$sourceext/i;
        move $filename, $newfilename;
    }

    &updateFileDisplay;

}

#replace string in file name
sub replaceString {

    &checkfilelist;

    return if ( scalar(@filelist) == 0 );

    if ( scalar(@filelist) == 1 ) {

        &fileErrMsg;
        return;
    }

    if ( $repsrcentry->get() eq "" ) {
        &dataErrMsg;
        return;
    }

    $srcstring    = $repsrcentry->get();
    $targetstring = $reptargetentry->get();

    if ( $reptargetentry->get() eq "" ) {
        my $changeQuestion;
        $changeQuestion = Tkx::tk___messageBox(
            -parent => $mw,
            -title  => "Confirm Removal of Text",
            -type   => "yesno",
            -icon   => "info",
            -message =>
              "Do you want to strip out \"$srcstring\"with no replacement?",
            -detail => "This operation cannot be undone.",
        );

        if ( $changeQuestion eq "no" ) {
            return;
        }
    }

    foreach (@filelist) {
        next unless -f $_;
        my $filename    = $_;
        my $newfilename = $filename;
        $newfilename =~ s/$srcstring/$targetstring/ig;
        move $filename, $newfilename;
    }
    &updateFileDisplay;
}

#undo replace string in filename
sub undoReplaceString {

    &checkfilelist;

    return if ( scalar(@filelist) == 0 );

    if ( scalar(@filelist) == 1 ) {

        &fileErrMsg;
        return;
    }

    if ( $repsrcentry->get() eq "" ) {
        &dataErrMsg;
        return;
    }

    if ( $reptargetentry->get() eq "" ) {
        my $sorryQuestion;
        $sorryQuestion = Tkx::tk___messageBox(
            -parent  => $mw,
            -title   => "Removal Operation Cannot Be Undone",
            -icon    => "info",
            -message => "You confirmed removing the string \"$srcstring.\"",
            -detail  => "This operation cannot be undone.",
        );
        return;
    }

    foreach (@filelist) {
        next unless -f $_;
        my $filename    = $_;
        my $newfilename = $filename;
        $newfilename =~ s/$targetstring/$srcstring/ig;
        move $filename, $newfilename;
    }
    &updateFileDisplay;
}

#replace string in file name
sub addString {

    &checkfilelist;

    return if ( scalar(@filelist) == 0 );

    if ( scalar(@filelist) == 1 ) {

        &fileErrMsg;
        return;
    }

    if ( $txtsrcentry->get() eq "" ) {
        &dataErrMsg;
        return;
    }

    $prependstring = $txtsrcentry->get();

    foreach (@filelist) {
        next unless -f $_;
        if ( $prepend == 0 ) {

            my $filename    = $_;
            my $newname     = $filename;
            my $basename    = basename($filename);
            my $dirname     = dirname($filename);
            my $newbasename = $basename;
            $newbasename = $prependstring . $basename;
            $newname =~ s/$basename/$newbasename/i;
            move $filename, $newname;
        }

        if ( $prepend == 1 ) {
            my $filename = $_;

            my @components = split( /\./, $filename );
            my $newname =
              $components[0] . $prependstring . "\." . $components[1];
            move $filename, $newname;
        }
    }
    &updateFileDisplay;
}

#replace string in file name
sub undoAddString {

    &checkfilelist;

    return if ( scalar(@filelist) == 0 );

    if ( scalar(@filelist) == 1 ) {

        &fileErrMsg;
        return;
    }

    if ( $txtsrcentry->get() eq "" ) {
        &dataErrMsg;
        return;
    }

    $prependstring = $txtsrcentry->get();

    foreach (@filelist) {
        next unless -f $_;

        my $fn    = $_;
        my $oldfn = $fn;
        $fn =~ s/$prependstring//;
        move $oldfn, $fn;

    }
    &updateFileDisplay;
}

#change modification time of files
sub changeTime {

    &checkfilelist;
    my $validation = &validateTime;
    return if ( $validation == 0 );
    return if ( scalar(@filelist) == 0 );

    my $changeQuestion;
    $changeQuestion = Tkx::tk___messageBox(
        -parent  => $mw,
        -title   => "Change Modification Time",
        -type    => "yesno",
        -icon    => "info",
        -message => "Do you want to change the file modification time?",
        -detail  => "This operation cannot be undone.",
    );

    if ( $changeQuestion eq "no" ) {
        return;
    }
    if ( $changeQuestion eq "yes" ) {
        my @datearray  = split( /\//, $datetime );
        my @clockarray = split( /\:/, $clocktime );
        my $hour;
        my $second = "00";
        if ( $timeofday == 1 ) {
            $hour = $clockarray[0] + 12;
        }
        else {
            $hour = $clockarray[0];
        }

        $modtime =
          timelocal( $second, $clockarray[1], $hour,
            $datearray[1], ( $datearray[0] - 1 ),
            $datearray[2] );
        print $modtime;

        foreach (@filelist) {

            utime $modtime, $modtime, $_;
        }

    }
}

#make sure time is formatted properly
sub validateTime {

    my $time = $timeentry->get();
    if ( $time =~ (/\d\d:\d\d/) ) {
        return 1;
    }
    else {
        $timeentry->delete( 0, "end" );

        Tkx::tk___messageBox(
            -parent  => $mw,
            -title   => "Incorrect Time Format",
            -icon    => "info",
            -message => "Incorrect Time Format",
            -detail  => "Please enter the time in HH:MM format, i.e. 12:30.",
        );
        return 0;
    }

}

#select directory from operations--this runs when drag and drop or from external script
sub selectFileSystem {

    my $fname = shift;
    my @name  = Tkx::SplitList($fname);
    my $path  = join "", @name;
    $folder = $path;

    if ( -f $folder ) {
        $listbox->delete( 0, "end" );
        @filelist = ();
        push( @filelist, $folder );
        foreach (@filelist) {
            $listbox->insert( "end", "$_" );
        }
    }

    if ( -d $folder ) {

        $listbox->delete( 0, "end" );
        @filelist = <"$folder/*">;
        foreach (@filelist) {
            $listbox->insert( "end", "$_" );
        }

        return $folder;
    }

    return $folder;
}

#"open"event from file or directory dragged onto icon
sub openDoc {

    my $fname = shift;
    &selectFileSystem($fname);
}

#show user help
sub showHelp {

    &Tkx::machelp__userhelp;
}

#initialize the app
sub startUp {

    $appname    = 'FileMorph';
    $appversion = '2.7';
    &Tkx::machelp__setAppName( $appname, $appversion );
    &Tkx::softwareupdate__setAppName($appname);
    &Tkx::softwareupdate__setVersion( $appname, $appversion );

    #check for app support directory
    our $prefdir = Tkx::xplat__appconfig("FileMorph");
    mkdir $prefdir unless ( -d $prefdir );
}

#check version of installed software
sub checkUpdate {

    &Tkx::softwareupdate__setIcon($icon);
    &Tkx::softwareupdate__checkingForUpdates;

    my $url    = "http://www.codebykevin.com/filemorph.ini";
    my $tmp    = new File::Temp( UNLINK => 1, SUFFIX => ".ini" );
    my $parser = get($url);
    print $tmp $parser;
    close $tmp;

    my $cfg = Config::IniFiles->new(
        -file          => $tmp,
        -allowcontinue => 1
      )
      or sub {

        Tkx::tk___messageBox(
            -parent  => $mw,
            -title   => "Error",
            -type    => "ok",
            -icon    => "warning",
            -message => "Cannot run operation on file",
            -detail =>
"An error occurred in retrieving update information.\nPlease try again later",
        );
        return;

      };

    my $versionnumber = $cfg->val( 'application', 'version' );
    $versionnumber = substr( $versionnumber, 0, -2 );
    my $minOS     = $cfg->val( 'application', 'osversion' );
    my $changelog = $cfg->val( 'application', 'changelog' );

    if ($IS_AQUA) {

        my $hostOS = `sw_vers -productVersion`;

        if ( $hostOS < $minOS ) {
            Tkx::tk___messageBox(
                -parent  => $mw,
                -title   => "Error",
                -type    => "ok",
                -icon    => "warning",
                -message => "Error!",
                -detail =>
"FileMorph is not supported on Mac OS X $hostOS. The minimum supported OS version is $minOS.",
            );
        }
    }

    if ( $appversion < $versionnumber ) {
        if ($IS_AQUA) {
            &Tkx::softwareupdate__updatePitchPerl( $appname,
                $versionnumber, $appversion, $changelog );
        }
        if ($IS_WINDOWS) {
            &Tkx::softwareupdate__winPitch( $appname, $versionnumber,
                $appversion );
        }

    }
    else {
        &Tkx::softwareupdate__upToDate();
    }
}

#-----AppleScript handlers------$

if ($IS_AQUA) {

    sub createAppleScriptCommands {
        Tkx::aem__installeventhandler( "CoKv", "fMdr", \&scriptSetDir );
        Tkx::aem__installeventhandler( "CoKv", "fMfl", \&scriptSetFile );
        Tkx::aem__installeventhandler( "CoKv", "oExt", \&scriptOldExt );
        Tkx::aem__installeventhandler( "CoKv", "nExt", \&scriptNewExt );
        Tkx::aem__installeventhandler( "CoKv", "rExt", \&scriptChangeExt );
        Tkx::aem__installeventhandler( "CoKv", "oTxt", \&scriptOldText );
        Tkx::aem__installeventhandler( "CoKv", "nTxt", \&scriptNewText );
        Tkx::aem__installeventhandler( "CoKv", "cTxt", \&scriptChangeText );
        Tkx::aem__installeventhandler( "CoKv", "beTx", \&scriptAddedText );
        Tkx::aem__installeventhandler( "CoKv", "afTx", \&scriptPrefixOrSuffix );
        Tkx::aem__installeventhandler( "CoKv", "adTx", \&scriptAddText );
        Tkx::aem__installeventhandler( "CoKv", "tmDa", \&scriptSetTime );
        Tkx::aem__installeventhandler( "CoKv", "cTme", \&scriptFileTime );
    }
}

sub scriptSetDir {
    my $path = shift;

    #join with spaces
    foreach (@_) {
        $path = $_ . ' ';
    }

    $folder = $path;

    if ( -f $folder ) {
        $listbox->delete( 0, "end" );
        @filelist = ();
        push( @filelist, $folder );
        foreach (@filelist) {
            $listbox->insert( "end", "$_" );
        }
    }

    if ( -d $folder ) {

        $listbox->delete( 0, "end" );
        @filelist = <"$folder/*">;
        foreach (@filelist) {
            $listbox->insert( "end", "$_" );
        }

        return $folder;
    }

    return $folder;
}

sub scriptOldExt {
    $sourceext = shift;
}

sub scriptNewExt {
    $targetext = shift;
}

sub scriptChangeExt {
    &changeExtension;
}

sub scriptOldText {
    $srcstring = shift;
}

sub scriptNewText {
    $targetstring = shift;
}

sub scriptChangeText {
    &replaceString;
}

sub scriptAddedText {
    $prependstring = shift;
}

sub scriptPrefixOrSuffix {
    $prepend = shift;
}

sub scriptAddText {
    &addString;
}

sub scriptSetTime {
    my $time;
    $time = shift;
    my @datearray = split( /\//, $time );
    $modtime =
      timelocal( "00", "00", "00", $datearray[1], ( $datearray[0] - 1 ),
        $datearray[2] );
}

sub scriptFileTime {
    return if ( scalar(@filelist) == 0 );
    foreach (@filelist) {
        utime $modtime, $modtime, $_;
    }
}

#------end of subroutines---------
